from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .alert_evidence import AlertEvidence
    from .file_evidence import FileEvidence
    from .process_evidence import ProcessEvidence

from .alert_evidence import AlertEvidence

@dataclass
class MalwareEvidence(AlertEvidence):
    # The OdataType property
    odata_type: Optional[str] = "#microsoft.graph.security.malwareEvidence"
    # The category property
    category: Optional[str] = None
    # The files property
    files: Optional[List[FileEvidence]] = None
    # The name property
    name: Optional[str] = None
    # The processes property
    processes: Optional[List[ProcessEvidence]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MalwareEvidence:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MalwareEvidence
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MalwareEvidence()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .alert_evidence import AlertEvidence
        from .file_evidence import FileEvidence
        from .process_evidence import ProcessEvidence

        from .alert_evidence import AlertEvidence
        from .file_evidence import FileEvidence
        from .process_evidence import ProcessEvidence

        fields: Dict[str, Callable[[Any], None]] = {
            "category": lambda n : setattr(self, 'category', n.get_str_value()),
            "files": lambda n : setattr(self, 'files', n.get_collection_of_object_values(FileEvidence)),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "processes": lambda n : setattr(self, 'processes', n.get_collection_of_object_values(ProcessEvidence)),
        }
        super_fields = super().get_field_deserializers()
        fields.update(super_fields)
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        super().serialize(writer)
        writer.write_str_value("category", self.category)
        writer.write_collection_of_object_values("files", self.files)
        writer.write_str_value("name", self.name)
        writer.write_collection_of_object_values("processes", self.processes)
    

